<?php
/*
Plugin Name: Twitter Blog
Plugin URI:  http://www.milkdogdesign.com/twitter_blog
Description: Twitter Blog will not only tweet about your blog post, but it will capture public replies to that tweet and add them as comments to your post. It also uses the<a href="http://bit.ly">bit.ly</a> API to link your link up tweeted blog link with your bit.ly account.
Version: 0.3.1
Author: Chris Mielke
Author URI: http://www.milkdogdesign.com/twitter_blog
*/

// Copyright (c) 2009 Chris Mielke. All rights reserved.
//
// Released under the GPL license
// http://www.opensource.org/licenses/gpl-license.php
//
// This is an add-on for WordPress
// http://wordpress.org/
//
//
// **********************************************************************
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
// **********************************************************************


class twitter_blog {
	function twitter_blog()
	{	
		//Gets Twitter username and password
		$this->twitter_username = get_option( 'tb_twitter_username' );
		$this->twitter_password = get_option( 'tb_twitter_password' );
		
		//Gets bit.ly username and API key
		$this->bitly_username = get_option( 'tb_bitly_username' );
		$this->bitly_api_key = get_option( 'tb_bitly_api_key' );
		
		//Twitter Blog Settings
		$this->tweet_comment_option = get_option( 'tb_tweet_comments' );
	}
	
	function install_twitter_blog()
	{
		//Sets the default settings
		update_option( 'tb_tweet_comments', 'on' );
	}
	
	function update_post_twitter_id($post_id, $twitter_status_id)
	{
		update_post_meta($post_id, '_twitter_status_id', $twitter_status_id);
			
		return true;

	}
	
	function update_comment_twitter_ids($post_id, $twitter_status_ids)
	{
		update_post_meta($post_id, '_twitter_comment_status_ids', $twitter_status_ids);
			
		return true;

	}

	function tweet_post($post_id)
	{
		if(get_post_meta($post_id, '_tweet_sent', true) != '1' )
		{
			$post = get_post($post_id);
			$post_link = get_permalink( $post_id );
			
			//Get bit.ly URL if API is set
			$bitly_options[CURLOPT_POSTFIELDS] = 'version=2.0.1&history=1&longUrl=' . $post_link . '&login=' . $this->bitly_username . '&apiKey=' . $this->bitly_api_key;
			$bitly_options[CURLOPT_RETURNTRANSFER] = true;
			
			$bitly_curl = curl_init( 'http://api.bit.ly/shorten' );
			curl_setopt_array($bitly_curl, $bitly_options);
			$bitly_response = curl_exec($bitly_curl);
			
			$bitly = json_decode($bitly_response);
			
			$post_link = urldecode($post_link);
			
			$bitly_link = $bitly->results->$post_link->shortUrl;
			
			curl_close($bitly_curl);
			
			// set options
			$options[CURLOPT_POSTFIELDS] = 'status=Blog Post: ' . urlencode($post->post_title) . ' ' . $bitly_link;
			$options[CURLOPT_USERPWD] = $this->twitter_username . ':' . $this->twitter_password;
			$options[CURLOPT_RETURNTRANSFER] = true;
	
			$curl = curl_init( 'http://twitter.com/statuses/update.json' );
			
			// set options
			curl_setopt_array($curl, $options);
			$response = curl_exec($curl);
			
			curl_close($curl);
			$twitter = json_decode($response);
			$this->update_post_twitter_id($post_id, $twitter->id);
	
			update_post_meta($post_id, '_tweet_sent', '1' );
		}
		return $post_id;
	}
	
	function insert_twitter_comment($tweet, $post_id)
	{
		//Gets time for comment
		$time = current_time( 'mysql', $gmt = 0); 
		
		//Creates comment array
		$data = array(
			'comment_post_ID' => $post_id,
			'comment_author' => '@' . $tweet->user->screen_name,
			'comment_author_email' => '',
			'comment_author_url' => 'http://twitter.com/' . $tweet->user->screen_name . '/status/' . $tweet->id,
			'comment_content' => str_replace( '@' . $this->twitter_username, '', $tweet->text),
			'comment_type' => '',
			'comment_parent' => 0,
			'user_ID' => 0,
			'comment_author_IP' => '',
			'comment_agent' => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.10) Gecko/2009042316 Firefox/3.0.10 (.NET CLR 3.5.30729)',
			'comment_date' => $time,
			'comment_date_gmt' => $time,
			'comment_approved' => 1,
		);
		
		//Inserts the tweet as a comment
		wp_insert_comment($data);
		
		
		//Sends a direct message 
		$options[CURLOPT_POSTFIELDS] = 'text=Your tweet has been added as a comment to @' . $this->twitter_username . '\'s blog - ' . get_permalink( $post_id ) . '&screen_name=' . $tweet->user->screen_name;
		$options[CURLOPT_USERPWD] = $this->twitter_username . ':' . $this->twitter_password;
		$options[CURLOPT_RETURNTRANSFER] = true;
		
		//Do CURL
		$curl = curl_init( 'http://twitter.com/direct_messages/new.json' );
		curl_setopt_array($curl, $options);
		$response = curl_exec($curl);
		curl_close($curl);
		
	}
	
	function check_twitter_comments()
	{
		//Only search for tweets if turned on
		if($this->tweet_comment_option == 'on' )
		{
			//Searches Twitter for mentions of the user.
			$options[CURLOPT_USERPWD] = $this->twitter_username . ':' . $this->twitter_password;
			$options[CURLOPT_RETURNTRANSFER] = true;
	
			$curl = curl_init( 'http://twitter.com/statuses/mentions.json?count=30' );

			curl_setopt_array($curl, $options);
			$response = curl_exec($curl);
			curl_close($curl);
			
			//Clear options
			unset($options);
			
			$json = json_decode($response);
				
			foreach($json as $tweet)
			{
				$options[CURLOPT_USERPWD] = $this->twitter_username . ':' . $this->twitter_password;
				$options[CURLOPT_RETURNTRANSFER] = true;
				
				//Do CURL
				$curl = curl_init('http://twitter.com/statuses/show/' . $tweet->id . '.json');
				curl_setopt_array($curl, $options);
				$status_response = curl_exec($curl);
				curl_close($curl);
				
				$status_json = json_decode($status_response);
					
				if(!empty($status_json->in_reply_to_status_id))
				{					  
					$twitter_posts = get_posts( 'meta_key=_twitter_status_id&meta_value=' . $status_json->in_reply_to_status_id);
				}
				
				
				
				if(count($twitter_posts) >= 1)
				{				
					//Should only be one result and gets that info
					foreach($twitter_posts as $temp_post_info)
					{
						$post_info = $temp_post_info;
					}
					
					
					//Check to see which Tweets have already been converted to comments
					$comment_ids = get_post_meta($post_info->ID, '_twitter_comment_status_ids', true);
					$comment_array = explode( ',', $comment_ids);
					if(!in_array($tweet->id, $comment_array) || count($comment_array) == 0)
					{
						$update_ids = (strlen($comment_ids) > 0) ? $comment_ids . ',' . $tweet->id : $tweet->id;
						
						$this->update_comment_twitter_ids($post_info->ID, $update_ids);
						
						//Inserts the Tweet as a comment
						$this->insert_twitter_comment($status_json, $post_info->ID);
					}
				}
			}
		}
	}
	
	function schedule_tweet_comments_cron() {
		wp_schedule_event(time(), 'hourly', 'hourly_twitter_comment_search' );
	}
	
	

	function unschedule_tweet_comments_cron() {
		wp_clear_scheduled_hook( 'hourly_twitter_comment_search' );
	}
	
	function twitter_blog_admin_menu_options() {
		echo( '<div class="wrap">
			<h2>Update Twitter Blog Settings</h2>' );
			
			wp_nonce_field( 'update-options' );
			
			if(isset($_POST['submit']) && $_POST['submit'] == 'Update Options' )
			{
				update_option( 'tb_twitter_username', $_POST['twitter_username']);
				$this->twitter_username = $_POST['twitter_username'];
				if(!empty($_POST['twitter_password']))
				{
					update_option( 'tb_twitter_password', $_POST['twitter_password'] );
				}
				update_option( 'tb_bitly_username', $_POST['bitly_username']);
				$this->bitly_username = $_POST['bitly_username'];
				update_option( 'tb_bitly_api_key', $_POST['bitly_api_key']);
				$this->bitly_api_key = $_POST['bitly_api_key'];
				
				update_option( 'tb_tweet_comments', $_POST['tweet_comments']);
				$this->tweet_comment_option = $_POST['tweet_comments'];
				echo( '<p class="updated">Settings Updated</p>' );
	
			}
			elseif(isset($_POST['submit']) && $_POST['submit'] == 'Create Comments' )
			{
				$this->check_twitter_comments();
				echo( '<p class="updated">Comments Updated</p>' );
			}
			
			//Checks if password is set for displaying
			$password_set = (!empty($this->twitter_password)) ? '' : '<span style="color: red;">Please set password</span>';
			
			//Checks the box for whether to Create comments from tweet replies.
			$tweet_comment_checkbox = ($this->tweet_comment_option == 'on' ) ? ' checked="checked"' : '';
		
			echo( '<form name="twitter_blog_settings" method="post" action="' . str_replace( '%7E', '~', $_SERVER['REQUEST_URI']) . '">
				<h3>Twitter Settings</h3>
				<p>
					<label for="twitter_username"><strong>Twitter Username</strong></label><br />
					<input type="text" size="40" name="twitter_username" value="' . $this->twitter_username . '" />
				</p>
				<p>
					<label for="twitter_password"><strong>Twitter Password</strong></label><br />
					<small>Only enter to update password</small><br />
					<input type="password" size="40" name="twitter_password" /> ' . $password_set . '
				</p>
				
				<h3>bit.ly Settings</h3>
				<p>
					<label for="bitly_username"><strong>bit.ly Username</strong></label> <input type="text" size="40" name="bitly_username" value="' . $this->bitly_username . '" />
				</p>
				<p>
					<label for="bitly_api_key"><strong>bit.ly API Key</strong></label><br />
					<small>Register for an account at <a href="http://bit.ly">bit.ly</a> and find your API Key on the <a href="http://bit.ly/account/">Acounts</a> page.</small><br />
					<input type="text" size="40" name="bitly_api_key" value="' . $this->bitly_api_key . '" />
				</p>
				<h3>Twitter Blog Settings</h3>
				<p>
					<label for="tweet_comments"><strong>Create comments from Tweet replies?</strong></label> <input type="checkbox" name="tweet_comments"' . $tweet_comment_checkbox  . ' />
				</p>
				<p>
					<input type="submit" name="submit" value="Update Options" />
				</p>
				<h3>Manually Create Comments from Tweet Replies</h>
				<p>
					<input type="submit" name="submit" value="Create Comments" />
				</p>' );
		
			echo( '</form>
		</div>' );
	}
}

//Set up Twitter Blog menu

//Creates the hooks and actions.
if(class_exists( 'twitter_blog' ))
{
	$plugin_twitter_blog = new twitter_blog();
}

if(isset($plugin_twitter_blog))
{
	function twitter_blog_admin_menu() 
	{
		global $plugin_twitter_blog;
		add_options_page( 'Twitter Blog Settings', 'Twitter Blog', 8, 'twitter-blog-menu', array(&$plugin_twitter_blog, 'twitter_blog_admin_menu_options' ));
	}
	register_activation_hook( __FILE__,  array(&$plugin_twitter_blog, 'install_twitter_blog' ));
	register_activation_hook(__FILE__, array(&$plugin_twitter_blog, 'schedule_tweet_comments_cron' ));
	register_deactivation_hook(__FILE__, array(&$plugin_twitter_blog, 'unschedule_tweet_comments_cron' ));
	
	add_action( 'admin_menu',  'twitter_blog_admin_menu' );	
	add_action( 'hourly_twitter_comment_search', array(&$plugin_twitter_blog, 'check_twitter_comments' ));
	add_action( 'publish_post', array(&$plugin_twitter_blog, 'tweet_post' ));	
	add_action( 'publish_post', array(&$plugin_twitter_blog, 'check_twitter_comments' ));	
	
	
}

?>